--*******************************************************************************************************************************
--
-- Description: This module contains a scatter gather DMA controller that interfaces to the PLDA EZDMA2 core.  One state machine
--				is used to manage the scatter gather operation while a second fetches the page descriptors through a linked list 
--				to set the correct address and transfer size for each burst.
--					
-- Rev History: 11-28-2018:	Added dma_ext_active input to support sharing of PLDA EZDMA2 resources.
-- 				01-10-2018:	Replaced asynchronous reset with synchronous reset.
--				11-15-2009:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

library UNISIM; 
use UNISIM.Vcomponents.all; 

entity dma_on_demand is
	port (
    -- Reset and clock.
	srst    			: in std_logic;						-- Active high synchronous reset in the system clock domain.
	clk 				: in std_logic;						-- System clock supplied by the endpoint core.

	-- DMA control and status signals.
	dma_init			: in std_logic;						-- Logic one causes the first descriptor to load when the controller is disabled.
	dma_rw				: in std_logic;						-- Logic zero selects a burst read operation, logic one selects write.
	dma_latency			: in std_logic_vector(1 downto 0);	-- Number of clock before data is valid in response to the dma_fifo_rd flag.
	dma_enable			: in std_logic;						-- Logic one enables the scatter gather controller.
	dma_go				: in std_logic;						-- Logic one indicates a burst transfer can be processed by the application logic.
	dma_abort			: in std_logic;						-- Logic one aborts any current DMA transaction.
    dma_current_page	: out std_logic_vector(9 downto 0);	-- Page number of the active DMA buffer.
    dma_current_burst	: out std_logic_vector(15 downto 0);-- Number of the active DMA burst within a page.
	dma_page_address	: in std_logic_vector(63 downto 0);	-- Physical address of the active DMA page.
	dma_descriptor		: in std_logic_vector(31 downto 0);	-- Descriptor assigned to the active DMA page.
	dma_fifo_wr			: out std_logic;					-- Logic one indicates that the PLDA EZDMA2 core is supplying data from a DMA read request.
	dma_fifo_rd			: out std_logic;					-- Logic one indicates that the PLDA EZDMA2 core is consuming data from a DMA write request.
	dma_marker			: out std_logic;					-- Logic one indicates that an interrupt is requested by the DMA channel.
	dma_busy			: out std_logic;					-- Logic one indicates that the DMA channel is currently active.
	dma_ext_active		: in std_logic;						-- Logic one indicates that the extended DMA controller is processing a request.

	-- Control and status signals connections to the PLDA EZDMA2 core.
	dma_wr			: in std_logic;							-- Logic one indicates that the PLDA EZDMA2 core is supplying data from a DMA read request.
	dma_wrchannel	: in std_logic;							-- Logic one indicates that data is available from this DMA channel.
	dma_rd			: in std_logic;							-- Logic one indicates that the PLDA EZDMA2 core is consuming data from a DMA write request.
	dma_rdchannel	: in std_logic;							-- Logic one indicates that data is needed on this DMA channel.
	dma_regin		: out std_logic_vector(127 downto 0);	-- Configuraton settings for the PLDA EZDMA2 core.
	dma_param 		: out std_logic_vector(23 downto 0);	-- DMA parameter settings for the PLDA EZDMA2 core.
	dma_control		: out std_logic_vector(5 downto 0);		-- DMA control settings for the PLDA EZDMA2 core.
	dma_status	 	: in std_logic;							-- DMA status from the PLDA EZDMA2 core.
	dma_fifocnt     : out std_logic_vector(12 downto 0)		-- FIFO count into the PLDA EZDMA2 core (not used).
	);
end dma_on_demand;

architecture structural of dma_on_demand is

	type dmastate is (dma_idle, burst_request, burst_start, burst_end, burst_complete, next_page);	-- Type definitions for the DMA state machine.
	signal dma_state		: dmastate;  					-- Current DMA machine state.
	signal burst_count	   	: unsigned(15 downto 0);		-- Counter used to track multiple bursts in a page.
	signal page_count	   	: unsigned(9 downto 0);			-- Counter used to track the number of pages.
	signal page_load_timer	: unsigned(1 downto 0);			-- Timer used to wait for the BRAM to respond.
	
begin
  	
--************************************************* STATE MACHINE TRANSITIONS **************************************************

	process (clk)
	begin
		if rising_edge(clk) then

			if srst = '1' then
			
				dma_state <= dma_idle;
				
			else
				
				-- Define each of the scatter gather state machine transitions.
				case dma_state is
				
					-- Remain idle while waiting for the application logic to arm and initiate a DMA transfer.
					when dma_idle =>
						-- Get the next descriptor if the burst count or burst size is set to zero in the current descriptor.
						if dma_enable = '1' and (dma_descriptor(15 downto 0) = x"0000" or dma_descriptor(29 downto 16) = "00000000000000") then
							dma_state <= next_page;
						-- Start a single burst transfer when commanded.
						elsif dma_enable = '1' and dma_go = '1' then
							dma_state <= burst_request;
						end if;
						
					-- Initiate the transfer of a single DMA burst.
					when burst_request =>
						-- Return to idle if the DMA is aborted by application logic.		
						if dma_abort = '1' then
							dma_state <= dma_idle;
						elsif dma_ext_active = '1' then
							dma_state <= burst_start;
						end if;
						
					-- Wait for the DMA burst transfer to start.
					when burst_start =>
						-- Return to idle if the DMA is aborted by application logic.		
						if dma_abort = '1' then
							dma_state <= dma_idle;
						elsif dma_status = '1' then
							dma_state <= burst_end;
						end if;
						
					-- Wait for the DMA burst transfer to complete.
					when burst_end =>
						-- Wait for the DMA status bit to transition from busy to idle.
						if dma_status = '0' then
							dma_state <= burst_complete;
						end if;
					
					-- Determine whether this was the last burst of the page.
					when burst_complete =>
						-- Advance to the next page if this was the last burst.
						if std_logic_vector(burst_count) = dma_descriptor(15 downto 0) then
							dma_state <= next_page;
						else
							dma_state <= dma_idle;
						end if;
					
					-- Request the next page from memory.
					when next_page =>
						if page_load_timer = "11" then
							dma_state <= dma_idle;
						end if;
						
					-- Define the default state to be dma_idle.
					when others => dma_state <= dma_idle;
					
				end case;
			end if;
		end if;
	end process;
 
--*************************************************** STATE MACHINE EVENTS *****************************************************

	process (clk)
	begin
		if rising_edge(clk) then

			-- Set the DMA control register based on the current state.
			if DMA_abort = '1' then
				-- Set the DMA abort command.
				dma_control <= "100000";
			elsif dma_state = burst_request and std_logic_vector(burst_count) = x"0000" then
				-- Update all of the DMA registers.
				dma_control <= "011111";
			elsif dma_state = burst_request and std_logic_vector(burst_count) /= x"0000" then
				-- Update only the parameter and size registers.
				dma_control <= "010100";
			else
				dma_control <= (others=>'0');
			end if;
			
			-- The burst counter starts at zero for each page and increments as bursts are completed within a page.
			-- Note: The burst count is reset to zero if requested (dma_init) when DMA transactions are disabled.	
			if dma_state = next_page or (dma_state = dma_idle and dma_enable = '0' and dma_init = '1') then
				-- Reset the burst counter when a new page is loaded.
				burst_count <= (others=>'0');
			elsif dma_state = burst_end and dma_status = '0' then
				-- Increment the burst counter when each burst completes.
				burst_count <= burst_count + x"0001";
			end if;
			
			-- The page counter starts at zero for each DMA buffer and increments as directed by the descriptor control bit.
			-- Note: The page count is reset to zero if requested (dma_init) when DMA transactions are disabled.
			if (dma_state = next_page and dma_descriptor(30) = '1' and page_load_timer = "00") or (dma_state = dma_idle and dma_enable = '0' and dma_init = '1') then
				-- Reset the page counter when a new page is loaded.
				page_count <= (others=>'0');
			elsif dma_state = next_page and page_load_timer = "00" then
				-- Increment the page counter when each new page is loaded.
				page_count <= page_count + "0000000001";
			end if;
			
			-- Set a timer each time a new page is requested from memory to compensate for latency through the BRAM.
			if dma_state /= next_page then
				-- Reset the page timer between page loads.
				page_load_timer <= (others=>'0');
			else
				-- Increment the page counter.
				page_load_timer <= page_load_timer + "01";
			end if;
			
		end if;
	end process;
				
--***************************************************** PORT ASSIGNMENTS *******************************************************
				
	-- Request a burst read or write of data.
	dma_param <= x"000" & "011" & DMA_rw & x"0" & dma_latency & "00";
	-- Perform a burst read or write to a page.
	dma_regin <= x"00000000" & x"0000" & "00" & dma_descriptor(29 downto 16) & dma_page_address;

	-- Do not throttle the DMA controller, the application logic should be ready to process a full burst when a transaction is requested.
    dma_fifocnt <= '1' & x"000";
    
	-- Create flags to indicate when a DMA write or read transaction is active for this particular channel.
    dma_fifo_wr <= dma_wrchannel and dma_wr;
    dma_fifo_rd <= dma_rdchannel and dma_rd;

	-- Assign external status signals.
    dma_current_burst <= std_logic_vector(burst_count);
    dma_current_page <= std_logic_vector(page_count);
    dma_marker <= '1' when dma_descriptor(31) = '1' and dma_state = next_page and page_load_timer = "00" else '0';
    dma_busy <= '0' when dma_state = dma_idle else '1';
    
end structural;
