--*******************************************************************************************************************************
--
-- Description: This package declares all of the unique data types used by the PCI Express model.
--					
-- Rev History: 11-28-2018:	Added support for sharing PLDA EZDMA2 resources.
-- 				01-10-2018:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use IEEE.math_real.all;

use work.model_specific_package.all;

package dma_package is

	-- Return the log2(N)-1, return 0 if N=0.
	function log2_index (x : natural) return natural;
	
	-- Define the multi-bus data types.
	type DMA_3 is array (8 downto 1) of std_logic_vector(2 downto 0);
	type DMA_10 is array (8 downto 1) of std_logic_vector(9 downto 0);
	type DMA_16 is array (8 downto 1) of std_logic_vector(15 downto 0);
	type DMA_AXI is array (8 downto 1) of std_logic_vector(AXI_WIDTH-1 downto 0);
	type FIFO_BYTE is array (1 downto 0) of std_logic_vector(0 downto 0);
	type FIFO_DATA is array (1 downto 0) of std_logic_vector(31 downto 0);
	type HECTA_4 is array (7 downto 1) of std_logic_vector(3 downto 0);
	type HECTA_6 is array (7 downto 1) of std_logic_vector(5 downto 0);
	type HECTA_13 is array (7 downto 1) of std_logic_vector(12 downto 0);
	type HECTA_24 is array (7 downto 1) of std_logic_vector(23 downto 0);
	type HECTA_128 is array (7 downto 1) of std_logic_vector(127 downto 0);
	type HECTA_AXI is array (7 downto 1) of std_logic_vector(AXI_WIDTH-1 downto 0);

	-- Define the DMA channel data types.
	type DMA_TYPE_1 is array (DMA_CHANNEL_COUNT downto 1) of std_logic;
	type DMA_TYPE_4 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(3 downto 0);
	type DMA_TYPE_6 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(5 downto 0);
	type DMA_TYPE_10 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(9 downto 0);
	type DMA_TYPE_13 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(12 downto 0);
	type DMA_TYPE_16 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(15 downto 0);
	type DMA_TYPE_24 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(23 downto 0);
	type DMA_TYPE_32 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(31 downto 0);
	type DMA_TYPE_64 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(63 downto 0);
	type DMA_TYPE_128 is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(127 downto 0);
	type DMA_TYPE_AXI is array (DMA_CHANNEL_COUNT downto 1) of std_logic_vector(AXI_WIDTH-1 downto 0);
	
	type DMA_EXT_TYPE_0 is array (DMA_ENGINE_COUNT downto 1) of std_logic_vector(0 downto 0);
	type DMA_EXT_TYPE_6 is array (DMA_ENGINE_COUNT downto 1) of std_logic_vector(5 downto 0);
	type DMA_EXT_TYPE_8 is array (DMA_ENGINE_COUNT downto 1) of std_logic_vector(7 downto 0);
	type DMA_EXT_TYPE_64 is array (DMA_ENGINE_COUNT downto 1) of std_logic_vector(63 downto 0);
	type DMA_EXT_SHAREU is array (DMA_ENGINE_COUNT downto 1) of unsigned(log2_index((DMA_CHANNEL_COUNT-1)/DMA_ENGINE_COUNT) downto 0);
	type DMA_EXT_PEND_6 is array (DMA_ENGINE_COUNT*natural(ceil(real(DMA_CHANNEL_COUNT)/real(DMA_ENGINE_COUNT))) downto 1) of std_logic_vector(5 downto 0);
	
    -- PLDA EZDMA licensed core.
	component plda_ezdma2 is
		port (
		clk                     : in    std_logic;
		rstn                    : in    std_logic;
		test_mode               : in    std_logic_vector (15 downto 0);
		user_lnk_up             : in    std_logic;
		s_axis_tx_tdata         : out   std_logic_vector (AXI_WIDTH-1 downto 0);
		s_axis_tx_tkeep         : out   std_logic_vector ((AXI_WIDTH/8)-1 downto 0);
		s_axis_tx_tlast         : out   std_logic;
		s_axis_tx_tvalid        : out   std_logic;
		s_axis_tx_tready        : in    std_logic;
		s_axis_tx_tuser         : out   std_logic_vector (3 downto 0);
		tx_err_drop             : in    std_logic;
		tbuf_av                 : in    std_logic_vector (5 downto 0);
		m_axis_rx_tdata         : in    std_logic_vector (AXI_WIDTH-1 downto 0);
		m_axis_rx_tkeep         : in    std_logic_vector ((AXI_WIDTH/8)-1 downto 0);
		m_axis_rx_tlast         : in    std_logic;
		m_axis_rx_tvalid        : in    std_logic;
		m_axis_rx_tuser         : in    std_logic_vector (21 downto 0);
		m_axis_rx_tready        : out   std_logic;
		rx_rnp_ok               : out   std_logic;
		cfg_err_cor             : out   std_logic;
		cfg_err_cpl_abort       : out   std_logic;
		cfg_err_cpl_timeout     : out   std_logic;
		cfg_err_cpl_unexpect    : out   std_logic;
		cfg_err_ecrc            : out   std_logic;
		cfg_err_posted          : out   std_logic;
		cfg_err_tlp_cpl_header  : out   std_logic_vector (47 downto 0);
		cfg_err_ur              : out   std_logic;
		cfg_err_cpl_rdy         : in    std_logic;
		cfg_err_locked          : out   std_logic;
		cfg_interrupt           : out   std_logic;
		cfg_interrupt_rdy       : in    std_logic;
		cfg_pcie_link_state     : in    std_logic_vector (2 downto 0);
		cfg_interrupt_assert    : out   std_logic;
		cfg_interrupt_di        : out   std_logic_vector (7 downto 0);
		cfg_interrupt_do        : in    std_logic_vector (7 downto 0);
		cfg_interrupt_mmenable  : in    std_logic_vector (2 downto 0);
		cfg_interrupt_msienable : in    std_logic;
		cfg_trn_pending         : out   std_logic;
		cfg_bus_number          : in    std_logic_vector (7 downto 0);
		cfg_device_number       : in    std_logic_vector (4 downto 0);
		cfg_function_number     : in    std_logic_vector (2 downto 0);
		cfg_status              : in    std_logic_vector (15 downto 0);
		cfg_command             : in    std_logic_vector (15 downto 0);
		cfg_dstatus             : in    std_logic_vector (15 downto 0);
		cfg_dcommand            : in    std_logic_vector (15 downto 0);
		cfg_lstatus             : in    std_logic_vector (15 downto 0);
		cfg_lcommand            : in    std_logic_vector (15 downto 0);
		int_request             : in    std_logic;
		int_ack                 : out   std_logic;
		int_msgnum              : in    std_logic_vector (4 downto 0);
		cfg_prmcsr              : out   std_logic_vector (31 downto 0);
		cfg_devcsr              : out   std_logic_vector (31 downto 0);
		cfg_linkcsr             : out   std_logic_vector (31 downto 0);
		cfg_msicsr              : out   std_logic_vector (15 downto 0);
		cfg_ltssm               : out   std_logic_vector (4 downto 0);
		slv_dataout             : out   std_logic_vector (AXI_WIDTH-1 downto 0);
		slv_bytevalid           : out   std_logic_vector ((AXI_WIDTH/8)-1 downto 0);
		slv_bytecount           : out   std_logic_vector (12 downto 0);
		slv_dwcount             : out   std_logic_vector (10 downto 0);
		slv_addr                : out   std_logic_vector (63 downto 0);
		slv_bar                 : out   std_logic_vector (6 downto 0);
		slv_readreq             : out   std_logic;
		slv_cpladdr             : out   std_logic_vector (31 downto 0);
		slv_cplparam            : out   std_logic_vector (7 downto 0);
		slv_writereq            : out   std_logic;
		slv_write               : out   std_logic;
		slv_lastwrite           : out   std_logic;
		slv_io                  : out   std_logic;
		slv_accept              : in    std_logic;
		slv_abort               : in    std_logic;
		slv_ur                  : in    std_logic;
		dma_rd                  : out   std_logic;
		dma_rdaddr              : out   std_logic_vector (15 downto 0);
		dma_rdchannel           : out   std_logic_vector (15 downto 0);
		dma_rddata              : in    std_logic_vector (AXI_WIDTH-1 downto 0);
		dma_wr                  : out   std_logic;
		dma_wraddr              : out   std_logic_vector (15 downto 0);
		dma_wrchannel           : out   std_logic_vector (15 downto 0);
		dma_wrdata              : out   std_logic_vector (AXI_WIDTH-1 downto 0);
		dma_wrbytevalid         : out   std_logic_vector ((AXI_WIDTH/8)-1 downto 0);
		dma0_regin              : in    std_logic_vector (127 downto 0);
		dma0_regout             : out   std_logic_vector (127 downto 0);
		dma0_param              : in    std_logic_vector (23 downto 0);
		dma0_control            : in    std_logic_vector (5 downto 0);
		dma0_status             : out   std_logic_vector (3 downto 0);
		dma0_fifocnt            : in    std_logic_vector (12 downto 0);
		dma1_regin              : in    std_logic_vector (127 downto 0);
		dma1_regout             : out   std_logic_vector (127 downto 0);
		dma1_param              : in    std_logic_vector (23 downto 0);
		dma1_control            : in    std_logic_vector (5 downto 0);
		dma1_status             : out   std_logic_vector (3 downto 0);
		dma1_fifocnt            : in    std_logic_vector (12 downto 0);
		dma2_regin              : in    std_logic_vector (127 downto 0);
		dma2_regout             : out   std_logic_vector (127 downto 0);
		dma2_param              : in    std_logic_vector (23 downto 0);
		dma2_control            : in    std_logic_vector (5 downto 0);
		dma2_status             : out   std_logic_vector (3 downto 0);
		dma2_fifocnt            : in    std_logic_vector (12 downto 0);
		dma3_regin              : in    std_logic_vector (127 downto 0);
		dma3_regout             : out   std_logic_vector (127 downto 0);
		dma3_param              : in    std_logic_vector (23 downto 0);
		dma3_control            : in    std_logic_vector (5 downto 0);
		dma3_status             : out   std_logic_vector (3 downto 0);
		dma3_fifocnt            : in    std_logic_vector (12 downto 0);
		dma4_regin              : in    std_logic_vector (127 downto 0);
		dma4_regout             : out   std_logic_vector (127 downto 0);
		dma4_param              : in    std_logic_vector (23 downto 0);
		dma4_control            : in    std_logic_vector (5 downto 0);
		dma4_status             : out   std_logic_vector (3 downto 0);
		dma4_fifocnt            : in    std_logic_vector (12 downto 0);
		dma5_regin              : in    std_logic_vector (127 downto 0);
		dma5_regout             : out   std_logic_vector (127 downto 0);
		dma5_param              : in    std_logic_vector (23 downto 0);
		dma5_control            : in    std_logic_vector (5 downto 0);
		dma5_status             : out   std_logic_vector (3 downto 0);
		dma5_fifocnt            : in    std_logic_vector (12 downto 0);
		dma6_regin              : in    std_logic_vector (127 downto 0);
		dma6_regout             : out   std_logic_vector (127 downto 0);
		dma6_param              : in    std_logic_vector (23 downto 0);
		dma6_control            : in    std_logic_vector (5 downto 0);
		dma6_status             : out   std_logic_vector (3 downto 0);
		dma6_fifocnt            : in    std_logic_vector (12 downto 0);
		dma7_regin              : in    std_logic_vector (127 downto 0);
		dma7_regout             : out   std_logic_vector (127 downto 0);
		dma7_param              : in    std_logic_vector (23 downto 0);
		dma7_control            : in    std_logic_vector (5 downto 0);
		dma7_status             : out   std_logic_vector (3 downto 0);
		dma7_fifocnt            : in    std_logic_vector (12 downto 0)
		);
	end component;
	
end dma_package;

package body dma_package is

	-- Return the log2(N)-1, return 0 if N=0.
	function log2_index (x : natural) return natural is
	begin
		if x = 0 then
			return 0;
		else
			return natural(ceil(log2(real(x))));
		end if;
	end log2_index;

end dma_package;
