--*******************************************************************************************************************************
--
-- Description: This core is used in place of qdr_sram as an interface stub for SRAM chips that are not driven by the FPGA
--				application logic.  This stub will hold the SRAM in a minimum power state.
--
-- Rev History: 08-04-2016:	Changed the name of qdriip_sys_clk to qdriip_clk to distinguish spread spectrum clock name.
-- 				07-14-2015:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all; 

use work.model_specific_package.all;
use work.transceiver_package.all;

entity qdr_sram is
	port (
	-- Reset, clocks and enable supplied by the application logic.
	rst         		: in std_logic;		-- Active high reset, logic one places all registers in a reset state.	
	qdriip_clk			: in std_logic;		-- QDR II+ fixed frequency or spread spectrum clock.
	clk200 				: in std_logic;		-- 200 MHz clock on global clock buffer.
	dll_on      		: in std_logic;		-- Logic one enables the delay-locked loop (DLL) internal to the SRAM.
	
	-- Write transaction interface signals.
	wr_clk       		: in std_logic;		-- Clock assigned to the write port.
	wr_rdy       		: out std_logic;	-- Logic one indicates that the write port is ready to accept another command.
	wr_cmd        		: in std_logic;		-- Logic one commits a write cycle to the SRAM using the current address and data.
	wr_add      		: in std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address input to the write port. 
	wr_data     		: in std_logic_vector(71 downto 0);	-- Data input to the write port.
	wr_bw	    		: in std_logic_vector(7 downto 0);	-- Active low byte enables [B7 ... B0] to the write port.
	
	-- Read transaction interface signals.
	rd_clk       		: in std_logic;		-- Clock assigned to the read port.
	rd_rdy        		: out std_logic;	-- Logic one indicates that the read port is ready to accept another command.
	rd_cmd        		: in std_logic;		-- Logic one commits a read cycle to the SRAM using the current address.
	rd_add      		: in std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address input to the read port.
	rd_data      		: out std_logic_vector(71 downto 0);				-- Data output from the read port.
	rd_vld	    		: out std_logic;	-- Logic one indicates that data is available from the requested read operation.
	rd_ack       		: in std_logic;	 	-- Logic one releases the current data and allows the next requested value to show through.
											-- The data ready (rd_vld) flag will remain high if new data is available on the next clock.
									
	-- Clock lock command and status signals.
	dll_rst_start		: in std_logic; 	-- Connect to the ready output of the IDELAY CONTROLLER.
	clk_lck_status		: out std_logic; 	-- Logic one indicates that clocks are locked.
	
	-- The following signals connect to a QDR II+ SRAM device through FPGA pins.
	qdriip_cq			: in  std_logic;						-- QDR II+ SRAM echo clock.
	qdriip_q			: in  std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous read data outputs.
	qdriip_qvld			: in  std_logic;						-- QDR II+ SRAM read data valid flag.
	qdriip_k			: out std_logic;						-- QDR II+ SRAM input clock.
	qdriip_d			: out std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous write data inputs.
	qdriip_sa			: out std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- QDR II+ SRAM synchronous address inputs.
	qdriip_w_n			: out std_logic;						-- QDR II+ SRAM synchronous active low write enable.
	qdriip_r_n			: out std_logic;						-- QDR II+ SRAM synchronous active low read enable.
	qdriip_bw_n			: out std_logic_vector(1 downto 0);		-- QDR II+ SRAM synchronous byte enables.
	qdriip_dll_off_n	: out std_logic							-- QDR II+ SRAM active low DLL disable.
	);	
end qdr_sram;

architecture behavioral of qdr_sram is

begin

--******************************************* ASSIGN CONSTANTS TO ALL OUTPUT PORTS *********************************************

	wr_rdy <= '0';
	rd_rdy <= '0';
	rd_data <= (others => '0');
	rd_vld <= '0';
	clk_lck_status <= '1';

	qdriip_k <= qdriip_cq and qdriip_qvld;
	qdriip_d <= qdriip_q;
	qdriip_sa <= (others => '0');
	qdriip_w_n <= '1';
	qdriip_r_n <= '1';
	qdriip_bw_n <= (others => '1');
	qdriip_dll_off_n <= '0';    
	
end behavioral;	